# -*- coding: utf-8 -*-#
#!/usr/bin/env python
# controller.py
import model as db

#----------------------------------------------------------------------
def addRecord(session, data):
    """
    Data should be a tuple of two dictionaries in the following format:

    ("author":{"first_name":"John", "last_name":"Doe"},
     "book":{"title":"Some book", "isbn":"1234567890",
             "publisher":"Packt"}
    )
    """
    book = db.Book()
    book.title = data["book"]["title"]
    book.isbn = data["book"]["isbn"]
    book.publisher = data["book"]["publisher"]
    author = db.Person()
    author.first_name = data["author"]["first_name"]
    author.last_name = data["author"]["last_name"]
    book.person = author

    # commit data to database
    session.add(book)
    session.commit()

#----------------------------------------------------------------------
def connectToDatabase():
    """
    Connect to our SQLite database and return a Session object
    """

    engine = db.sa.create_engine("sqlite:///devdata.db", echo=True)
    db.init_model(engine)
    session = db.DBSession()
    setupDatabase(engine)
    return session

#----------------------------------------------------------------------
def convertResults(results):
    """
    Convert results to OlvBook objects
    """
    print
    books = []
    for record in results:
        author = "%s %s" % (record.person.first_name,
                            record.person.last_name)
        book = OlvBook(record.id, record.title, author,
                       record.isbn, record.publisher,
                       record.person.last_name,
                       record.person.first_name
                       )
        books.append(book)
    return books

#----------------------------------------------------------------------
def deleteRecord(session, idNum):
    """
    Delete a record from the database
    """
    record = session.query(db.Book).filter_by(id=idNum).one()
    session.delete(record)
    session.commit()

#----------------------------------------------------------------------
def editRecord(session, idNum, row):
    """
    Edit a record
    """
    record = session.query(db.Book).filter_by(id=idNum).one()
    print
    record.title = row["title"]
    record.person.first_name = row["first_name"]
    record.person.last_name = row["last_name"]
    record.isbn = row["isbn"]
    record.publisher = row["publisher"]
    session.add(record)
    session.commit()

#----------------------------------------------------------------------
def getAllRecords(session):
    """
    Get all records and return them
    """
    result = session.query(db.Book).all()
    return result

#----------------------------------------------------------------------
def searchRecords(session, filterChoice, keyword):
    """
    Searches the database based on the filter chosen and the keyword
    given by the user
    """
    if filterChoice == "Author":
        qry = session.query(db.Person)
        result = qry.filter(db.Person.first_name.contains('%s' % keyword)).all()
        records = []
        for record in result:
            for book in record.books:
                records.append(book)
        result = records
        print result
    elif filterChoice == "Title":
        qry = session.query(db.Book)
        result = qry.filter(db.Book.title.contains('%s' % keyword)).all()
    elif filterChoice == "ISBN":
        qry = session.query(db.Book)
        result = qry.filter(db.Book.isbn.contains('%s' % keyword)).all()
    else:
        qry = session.query(db.Book)
        result = qry.filter(db.Book.publisher.contains('%s' % keyword)).all()
    return result

#----------------------------------------------------------------------
def setupDatabase(engine):
    """Setup the db, note that this will not upgrade already existing tables"""
    db.metadata.create_all(engine)
    print "db created"
